/*
 * tunerctl.c: Switch frequency on tuner
 *
 * Copyright (c) 1994-96 Martin Buck  <martin-2.buck@student.uni-ulm.de>
 * Read COPYING for more information
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <sys/vtx.h>
#include "safe_malloc.h"
#include "vtx_assert.h"
#include "misc.h"
#include "tuner.h"


int
main(int argc, char *argv[]) {
  tuner_info_t tuner_info;
  int optchr, new_freq, freq = -1;

  if ((smalloc_progname = strrchr(argv[0], '/'))) {
    smalloc_progname++;
  } else {
    smalloc_progname = argv[0];
  }

  while ((optchr = getopt(argc, argv, "D:?")) != -1) {
    switch (optchr) {
      case 'D':
        tuner_device = optarg;
      break;
      case '?':
        printf("tunerctl version " VTXVERSION ", Copyright 1994-96 Martin Buck\n\n"
               "Usage: %s [options] [<frequency>]\n"
               "Frequency syntax:\n"
               "  xxx[.yyy][M[Hz]]\n"
               "  xxxk[Hz]\n"
               "Options:\n"
               "  -D <device>   Set device to use (default: /dev/tuner)\n"
               "  -\\?           Show this help\n", smalloc_progname);
        exit(1);
      break;
      default:
        assert(0);
      break;
    }
  }

  if (optind == argc - 1) {
    char unit[16] = "";
    int count;
    double tmp_freq;
    
    count = sscanf(argv[optind], " %lf %15s ", &tmp_freq, unit);
    if (count < 1) {
      fprintf(stderr, "%s: invalid argument: %s\n", smalloc_progname, argv[optind]);
      exit(1);
    }
    if (unit[0] != '\0' && strcasecmp(unit, "m") && strcasecmp(unit, "mhz") &&
        strcasecmp(unit, "k") && strcasecmp(unit, "khz")) {
      fprintf(stderr, "%s: invalid frequency unit: %s\n", smalloc_progname, unit);
      exit(1);
    }
    freq = (tolower(unit[0]) == 'k' ? (int)tmp_freq : (int)(tmp_freq * 1000));
    if (freq < 0) {
      fprintf(stderr, "%s: frequency out of range: %.3f MHz\n", smalloc_progname, freq / 1000.0);
      exit(1);
    }
  } else if (optind != argc) {
    fprintf(stderr, "%s: too many arguments (try `%s -\\?' to get help)\n", smalloc_progname,
        smalloc_progname);
    exit(1);
  }

  switch (tuner_open(REQ_MAJOR, REQ_MINOR, &tuner_info)) {
    case TUNOK:
    break;
    case TUNEVERSION:
      fprintf(stderr, "%s: can't open tuner-device: incompatible driver version (need %d.%d)\n",
          smalloc_progname, REQ_MAJOR, REQ_MINOR);
      exit(1);
    break;
    default:
      fprintf(stderr, "%s: can't open tuner-device %s: %s\n", smalloc_progname, tuner_device,
          strerror(errno));
      exit(1);
    break;
  }

  if (freq >= 0) {
    switch (tuner_setfreq(freq)) {
      case TUNOK:
      break;
      case TUNEINVAL:
        fprintf(stderr, "%s: frequency out of range: %.3f MHz\n", smalloc_progname, freq / 1000.0);
        exit(1);
      break;
      default:
        fprintf(stderr, "%s: can't set frequency: %s\n", smalloc_progname, strerror(errno));
        exit(1);
      break;
    }
  }
  
  if ((new_freq = tuner_getfreq()) < 0) {
    fprintf(stderr, "%s: can't read%s frequency: %s\n", smalloc_progname, (freq >= 0 ? " new" : ""),
        strerror(errno));
    exit(1);
  }
  printf("%s tuning frequency: %.3f MHz\n", (freq >= 0 ? "new" : "current"), new_freq / 1000.0);
  exit(0);
}
